--[[	PhantomPlates BuffFrames Module
	by SDPhantom
	https://www.wowinterface.com/forums/member.php?u=34145
	https://www.curseforge.com/members/sdphantomgamer/projects	]]
--------------------------------------------------------------------------

--------------------------
--[[	Namespace	]]
--------------------------
local AddOn=select(2,...);

--------------------------
--[[	Library Loader	]]
--------------------------
local LibClassicDurations=AddOn.IsVanillaClient and LibStub and LibStub("LibClassicDurations",true);
local LibAuraTypes=LibStub and LibStub("LibAuraTypes",true);

--------------------------
--[[	Constants	]]
--------------------------
local BuffFilter_Player="HELPFUL|INCLUDE_NAME_PLATE_ONLY";--	"HELPFUL|INCLUDE_NAME_PLATE_ONLY"
local BuffFilter_Hostile="HARMFUL|INCLUDE_NAME_PLATE_ONLY";--	"HARMFUL|INCLUDE_NAME_PLATE_ONLY"
local BuffFilter_Friendly="HARMFUL|RAID";--	"HARMFUL|RAID"

local BuffFilter_Order={"HELPFUL","HARMFUL"};
local BuffFilter_AllyPriority="SILENCE";--	Default used by ClassicAuraDurations portraits
local BuffFilter_EnemyPriority="SILENCE";--	Default used by ClassicAuraDurations portraits

local BuffFrame_Offsets={
	[true]={4,0};--	{4,-3};
	[false]={4,-12};
};
local CCButton_NormalOffset={0,-6};
local CCButton_EliteOffset={12,-6};
local BuffButton_Size={20,14};
local BuffButton_Border=1;
local BuffButton_Spacing=4;

----------------------------------
--[[	Helper Functions	]]
----------------------------------
local AuraIterator; do--	AuraIterator(unit,filter)
	local function AuraHelper(index,...) return (...) and index or nil,...; end
	local function NextAura(state,index) index=(index or 0)+1; return AuraHelper(index,UnitAura(state[1],index,state[2])); end
	function AuraIterator(unit,filter) return NextAura,{unit,filter},nil; end
end

------------------
--[[	Hooks	]]
------------------
local BuffFrames={};

local function CreateBuffButton(container,...)
	local width,height=unpack(BuffButton_Size);
	local buff=CreateFrame("Frame",nil,container);
	if (...) then buff:SetPoint(...); end
	buff:SetSize(width,height);

	buff.Border=buff:CreateTexture(nil,"BACKGROUND");
	buff.Border:SetAllPoints();
	buff.Border:SetColorTexture(0,0,0);

	buff.Icon=buff:CreateTexture(nil,"ARTWORK");
	buff.Icon:SetPoint("CENTER");
	buff.Icon:SetSize(width-BuffButton_Border*2,height-BuffButton_Border*2);
	buff.Icon:SetTexCoord(0.05,0.95,0.1,0.6);

	buff.Stealable=buff:CreateTexture(nil,"OVERLAY");
	buff.Stealable:SetAllPoints();
	buff.Stealable:SetTexture("Interface\\TargetingFrame\\UI-TargetingFrame-Stealable");
	buff.Stealable:Hide();

	buff.Cooldown=CreateFrame("Cooldown",nil,buff,"CooldownFrameTemplate");
	buff.Cooldown:SetPoint("CENTER",0,-1);
	buff.Cooldown:SetDrawEdge(true);
	buff.Cooldown:SetHideCountdownNumbers(true);
	buff.Cooldown:SetReverse(true);
	buff.Cooldown:Hide();

	buff.Count=buff:CreateFontString(nil,"OVERLAY","NumberFontNormalSmall");
	buff.Count:SetPoint("BOTTOMRIGHT",3,-2);
	buff.Count:SetJustifyH("RIGHT");

	return buff;
end

local function UpdateCCBuffAnchor(button,unit)
	local classification=AddOn.Options.Classification_ShowElite and (not UnitIsPlayer(unit)) and UnitClassification(unit);
	button:SetPoint("LEFT",button:GetParent(),"RIGHT",unpack(
		(
			classification=="worldboss"
			or classification=="elite"
			or classification=="rareelite"
		) and CCButton_EliteOffset or CCButton_NormalOffset
	));
end

local function UpdateBuffs(self,unit)
	local showbuffs=AddOn.Options.BuffFrames_ShowBuffs;
	local showcc=AddOn.Options.BuffFrames_ShowCC;

	local isplayer=UnitIsUnit("player",unit);
	local ishostile; if not isplayer then--	Recycled logic from Modern WoW's NamePlates
		local reaction=UnitReaction("player",unit);
		ishostile=(reaction and reaction<=4);
	end

	local priorityfilter=LibAuraTypes and AddOn.Options.BuffFrames_ShowCC
		and (ishostile
			and LibAuraTypes.GetAuraTypePriority(BuffFilter_EnemyPriority,"ENEMY")
			or LibAuraTypes.GetAuraTypePriority(BuffFilter_AllyPriority,"ALLY")
		) or nil;

	local buffindex=1;
	local cc_icon,cc_count,cc_bufftype,cc_duration,cc_expires;
	for _,filter in ipairs(BuffFilter_Order) do
		local isdebuff=filter:find("HARMFUL") and true or false;--	We may change tags at a later date, this ensures some futureproofing

		for index,name,icon,count,bufftype,duration,expires,caster,stealable,_,spellid in AuraIterator(unit,filter) do
			if LibClassicDurations and duration<=0 and expires<=0 then
				local newduration,newexpires=LibClassicDurations:GetAuraDurationByUnit(unit,spellid,caster);
				if newduration and newexpires then duration,expires=newduration,newexpires; end
			end

			if name and icon then
				if showbuffs and (caster=="player" or caster=="pet" or caster=="vehicle" or (not isplayer and stealable)) then
					local buff=self.Buffs[buffindex];
					if not buff then
						buff=CreateBuffButton(self,"BOTTOMLEFT",(buffindex-1)*(BuffButton_Size[1]+BuffButton_Spacing),0);
						self.Buffs[buffindex]=buff;
					end

					buff.Icon:SetTexture(icon);
					buff.Border:SetShown(isdebuff);
					if isdebuff then buff.Border:SetColorTexture(ColorMixin.GetRGB(bufftype and DebuffTypeColor[bufftype] or DebuffTypeColor.none)); end
					buff.Stealable:SetShown(not isplayer and stealable);
					buff.Count:SetText(count>1 and count or nil);

					CooldownFrame_Set(buff.Cooldown,expires-duration,duration,duration>0,true);

					buff:Show();
					buffindex=buffindex+1;
				end

				if showcc and priorityfilter and isdebuff then
					local priority=LibAuraTypes.GetAuraInfo(spellid,ishostile and "ENEMY" or "ALLY");
					if priority and priority>=priorityfilter and (not cc_expires or expires>cc_expires) then--	Show CC that expires last
						cc_icon,cc_count,cc_bufftype,cc_duration,cc_expires=icon,count,bufftype,duration,expires;
					end
				end
			end
		end
	end

	for i=buffindex,#self.Buffs do self.Buffs[i]:Hide(); end

	if cc_icon then
		local buff=self.CCBuff;
		if not buff then
			buff=CreateBuffButton(self:GetParent());
			self.CCBuff=buff;
		end

		UpdateCCBuffAnchor(buff,unit);

		buff.Icon:SetTexture(cc_icon);
		buff.Border:SetColorTexture(ColorMixin.GetRGB(cc_bufftype and DebuffTypeColor[cc_bufftype] or DebuffTypeColor.none));
		buff.Count:SetText(cc_count>1 and cc_count or nil);

		CooldownFrame_Set(buff.Cooldown,cc_expires-cc_duration,cc_duration,cc_duration>0,true);

		buff:Show();
	elseif self.CCBuff then self.CCBuff:Hide(); end

	local show=(buffindex>1);
	self:SetShown(show);
	if show then self:SetWidth((buffindex-1)*(BuffButton_Size[1]+BuffButton_Spacing)-BuffButton_Spacing); end
end

AddOn.OnNamePlateCreated:AddCallback(function(_,base)
	local unitframe=base.UnitFrame;
	if not (unitframe:IsForbidden() or BuffFrames[unitframe]) then
		local frame=CreateFrame("Frame",nil,unitframe);
		frame:SetSize(unpack(BuffButton_Size));

		frame.Buffs={};
		BuffFrames[unitframe]=frame;
	end
end);

AddOn.OnNamePlateAdded:AddCallback(function(base,unit)
	local unitframe=base.UnitFrame;
	local container=BuffFrames[unitframe];
	if container and (AddOn.Options.BuffFrames_ShowBuffs or AddOn.Options.BuffFrames_ShowCC) then
		if LibClassicDurations then LibClassicDurations:RegisterFrame(container); end
--		UpdateBuffs(container,unit);--	Handled by our NamePlateDriverFrame:OnUnitAuraUpdate() hook
		container:SetPoint("BOTTOMLEFT",unitframe,"TOPLEFT",unpack(BuffFrame_Offsets[unitframe.name:IsShown()]));
	end
end);

if LibClassicDurations then AddOn.OnNamePlateRemoved:AddCallback(function(base)
	local container=BuffFrames[base.UnitFrame];
	if container then LibClassicDurations:UnregisterFrame(container); end
end); end

hooksecurefunc(NamePlateDriverFrame,"OnUnitAuraUpdate",function(_,unit)
	if not (AddOn.Options.BuffFrames_ShowBuffs or AddOn.Options.BuffFrames_ShowCC) then return; end
	for unitframe,container in pairs(BuffFrames) do
		local unitframeunit=unitframe.displayedUnit;
		if unitframeunit and UnitIsUnit(unitframeunit,unit) then
			UpdateBuffs(container,unit);
		end
	end
end);

----------------------------------
--[[	Options Defaults	]]
----------------------------------
AddOn.Options.BuffFrames_ShowBuffs=true;
AddOn.Options.BuffFrames_ShowCC=true;

--------------------------
--[[	Options Layout	]]
--------------------------
local SubPanel=AddOn.OptionsPanel:CreateSubPanel(nil);
SubPanel:SetHeight(56);

local ShowBuffsButton=CreateFrame("CheckButton",nil,SubPanel,"UICheckButtonTemplate");
ShowBuffsButton:SetPoint("TOPLEFT",4,0);
ShowBuffsButton.text:SetText(AddOn.Localization.OptionsSetting_BuffFrames_ShowBuffs);

local ShowCCButton=CreateFrame("CheckButton",nil,SubPanel,"UICheckButtonTemplate");
ShowCCButton:SetPoint("TOPLEFT",4,-24);
ShowCCButton.text:SetText(AddOn.Localization.OptionsSetting_BuffFrames_ShowCC);

--------------------------
--[[	Options Setup	]]
--------------------------
AddOn.OptionsPanel:RegisterControl("BuffFrames_ShowBuffs",ShowBuffsButton
	,nil
	,ShowBuffsButton.SetChecked
	,ShowBuffsButton.GetChecked
	,"OnClick"
);

AddOn.OptionsPanel:RegisterControl("BuffFrames_ShowCC",ShowCCButton
	,nil
	,ShowCCButton.SetChecked
	,ShowCCButton.GetChecked
	,"OnClick"
);

local function BuffFrames_OnOptionsUpdate()
	local enabled=(AddOn.Options.BuffFrames_ShowBuffs or AddOn.Options.BuffFrames_ShowCC);
	for unitframe,container in pairs(BuffFrames) do
		local unit=unitframe.displayedUnit; if unit then
			if enabled then UpdateBuffs(container,unit); else
				container:Hide();
				if container.CCBuff then container.CCBuff:Hide(); end
			end
		end
	end
end

AddOn.OptionsPanel:HookOption("BuffFrames_ShowBuffs",BuffFrames_OnOptionsUpdate);
AddOn.OptionsPanel:HookOption("BuffFrames_ShowCC",BuffFrames_OnOptionsUpdate);

AddOn.OptionsPanel:HookOption("Classification_ShowElite",function(_,_,val)
	if not AddOn.Options.BuffFrames_ShowCC then return; end
	for unitframe,container in pairs(BuffFrames) do
		local unit,buff=unitframe.displayedUnit,container.CCBuff;
		if unit and buff then UpdateCCBuffAnchor(buff,unit); end
	end
end);
