--[[	PhantomPlates PowerBars Module
	by SDPhantom
	https://www.wowinterface.com/forums/member.php?u=34145
	https://www.curseforge.com/members/sdphantomgamer/projects	]]
--------------------------------------------------------------------------

--------------------------
--[[	Namespace	]]
--------------------------
local AddOn=select(2,...);

----------------------------------
--[[	Import From Utilities	]]
----------------------------------
local AddOn_Utilities_UnitPowerBarColor=AddOn.Utilities.UnitPowerBarColor;

------------------------------------------
--[[	HealthBar Color Adjustments	]]
------------------------------------------
--	Lock friendly health bars to green, otherwise mana bars are hard to see
--DefaultCompactNamePlateFriendlyFrameOptions.colorHealthBySelection=false;

------------------
--[[	Hooks	]]
------------------
local PowerBars={};

local function NamePlate_UpdateHealthColorOverride(self)--	HealthBar color lock
	if	not (self.optionTable.colorHealthBySelection and UnitPlayerControlled(self.unit))
	or	CompactUnitFrame_IsTapDenied(self)
	or	((self.optionTable.allowClassColorsForNPCs or UnitIsPlayer(self.unit)) and RAID_CLASS_COLORS[select(2,UnitClass(self.unit))] and self.optionTable.useClassColors)
	then return false; end--	Allow CompactUnitFrame_UpdateHealthColor() to continue

	local r,g,b; if UnitCanAttack("player",self.unit) then--	Separate by PvP-enabled enemy players
		local faction=UnitFactionGroup("player");
		if UnitIsPVPFreeForAll("player") or (faction and faction~="Neutral" and UnitIsPVP("player")) then
			r,g,b=1,0,0;--	Hostile if we're PvP-flagged too
		else r,g,b=1,1,0; end--	Neutral; unit is PvP-flagged, but we aren't
	else r,g,b=0,1,0; end--	Default to friendly

	self.healthBar:SetStatusBarColor(r,g,b);
	self.healthBar.r,self.healthBar.g,self.healthBar.b=r,g,b;--	Update for CompactUnitFrame_UpdateHealthColor()

	return true;--	Signal we updated color ourselves
end

AddOn.OnNamePlateCreated:AddCallback(function(_,base)
	local unitframe=base.UnitFrame;
	if not (unitframe:IsForbidden() or PowerBars[unitframe]) then
		local bar=unitframe.healthBar:CreateTexture(nil,"BORDER",nil,0);
		bar:SetTexture("Interface/TargetingFrame/UI-TargetingFrame-BarFill");
		bar:SetDrawLayer("BORDER",1);
		bar:SetPoint("TOPLEFT",unitframe.healthBar,"LEFT");
		bar:SetPoint("BOTTOM");
		bar:SetWidth(unitframe.healthBar:GetWidth());

		if not AddOn.Options.PowerBars_Enabled then bar:Hide(); end
		unitframe.UpdateHealthColorOverride=NamePlate_UpdateHealthColorOverride;--	TBC+ hook (Vanilla hook below)
		PowerBars[unitframe]=bar;
	end
end);

--	These functions are shared with other UnitFrames
local function UpdatePower(self)
	local bar,displayedunit=PowerBars[self],self.displayedUnit;
	if not (bar and AddOn.Options.PowerBars_Enabled) then return; end

	local powertype=UnitPowerType(displayedunit);
	local cur,max=UnitPower(displayedunit,powertype),UnitPowerMax(displayedunit,powertype);
	if max>0 then
		local percent=cur/max;
		local width=percent*self.healthBar:GetWidth();
		if width>=1 then
			bar:Show();
			bar:SetWidth(width);
			bar:SetTexCoord(0,percent,0,1);
			return;
		end
	end
	bar:Hide();
end

local function UpdatePowerColor(self)
	if not (PowerBars[self] and AddOn.Options.PowerBars_Enabled) then return; end
	PowerBars[self]:SetVertexColor(AddOn_Utilities_UnitPowerBarColor(self.displayedUnit));
end

--	:OnAdded() calls CompactUnitFrame_UpdateAll(), which calls these anyway
hooksecurefunc("CompactUnitFrame_UpdateMaxPower",UpdatePower);
hooksecurefunc("CompactUnitFrame_UpdatePower",UpdatePower);
hooksecurefunc("CompactUnitFrame_UpdatePowerColor",UpdatePowerColor);

--	Hook for Vanilla Classic (TBC hook uses :UpdateHealthColorOverride() above)
if AddOn.IsVanillaClient then hooksecurefunc("CompactUnitFrame_UpdateHealthColor",function(self)
	if PowerBars[self] then NamePlate_UpdateHealthColorOverride(self); end
end); end

----------------------------------
--[[	Options Defaults	]]
----------------------------------
AddOn.Options.PowerBars_Enabled=true;

--------------------------
--[[	Options Layout	]]
--------------------------
local SubPanel=AddOn.OptionsPanel:CreateSubPanel(1);
SubPanel:SetHeight(32);

local EnableButton=CreateFrame("CheckButton",nil,SubPanel,"UICheckButtonTemplate");
EnableButton:SetPoint("TOPLEFT",4,0);
EnableButton.text:SetText(AddOn.Localization.OptionsSetting_PowerBars_Enabled);

--------------------------
--[[	Options Setup	]]
--------------------------
AddOn.OptionsPanel:RegisterControl("PowerBars_Enabled",EnableButton
	,nil
	,EnableButton.SetChecked
	,EnableButton.GetChecked
	,"OnClick"
);

AddOn.OptionsPanel:HookOption("PowerBars_Enabled",function(_,_,val)
	for unitframe,bar in pairs(PowerBars) do
		bar:SetShown(val);
		if unitframe.displayedUnit then
			UpdatePower(unitframe);
			UpdatePowerColor(unitframe);
		end
	end
end);
